# views.py
import logging  # ✅ 로그 추가
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework_simplejwt.views import TokenObtainPairView

from .serializers import RegisterSerializer, CustomTokenObtainPairSerializer

logger = logging.getLogger(__name__)  # ✅ 로그 추가

class RegisterView(APIView):
    def post(self, request):
        logger.info("RegisterView POST called. data=%s", request.data)  # ✅ 로그 추가
        serializer = RegisterSerializer(data=request.data)
        if serializer.is_valid():
            user = serializer.save()
            logger.info("User registered: %s", user.email if hasattr(user, "email") else str(user))  # ✅ 로그 추가
            return Response({"message": "User registered successfully."}, status=status.HTTP_201_CREATED)
        logger.warning("Registration failed: %s", serializer.errors)  # ✅ 로그 추가
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class MeView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        logger.debug("MeView GET called by user: %s", request.user)  # ✅ 로그 추가
        user = request.user
        serializer = RegisterSerializer(user)
        return Response(serializer.data)

    def put(self, request):
        logger.info("MeView PUT called by user: %s", request.user)  # ✅ 로그 추가
        user = request.user
        serializer = RegisterSerializer(user, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            logger.info("User profile updated: %s", request.user)  # ✅ 로그 추가
            return Response(serializer.data)
        logger.warning("Profile update failed: %s", serializer.errors)  # ✅ 로그 추가
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class CustomTokenObtainPairView(TokenObtainPairView):
    serializer_class = CustomTokenObtainPairSerializer

    def post(self, request, *args, **kwargs):
        logger.info("Login attempt: %s", request.data.get("email"))  # ✅ 로그 추가 (비밀번호 제외)
        return super().post(request, *args, **kwargs)


class SSHKeyUploadView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        logger.info("SSHKeyUploadView POST called by user: %s", request.user)  # ✅ 로그 추가
        private_key = request.data.get("private_key")
        key_name = request.data.get("key_name")

        if not private_key or not key_name:
            logger.warning("Missing SSH key or key name in request")  # ✅ 로그 추가
            return Response(
                {"error": "private_key와 key_name 모두 필요합니다."},
                status=status.HTTP_400_BAD_REQUEST
            )

        user = request.user
        try:
            user.save_private_key(private_key)
            user.encrypted_private_key_name = key_name
            user.save(update_fields=["encrypted_private_key", "encrypted_private_key_name"])
            logger.info("SSH key saved for user: %s", user)  # ✅ 로그 추가
            return Response({"message": "SSH key 저장 완료."}, status=201)
        except Exception as e:
            logger.exception("SSH key 저장 중 오류 발생")  # ✅ 로그 추가
            return Response({"error": f"암호화 또는 저장 실패: {str(e)}"}, status=500)

    def delete(self, request):
        logger.info("SSHKeyUploadView DELETE called by user: %s", request.user)  # ✅ 로그 추가
        user = request.user
        user.encrypted_private_key = None
        user.encrypted_private_key_name = None
        user.last_used_at = None
        user.save(update_fields=["encrypted_private_key", "encrypted_private_key_name", "last_used_at"])
        logger.info("SSH key deleted for user: %s", user)  # ✅ 로그 추가
        return Response({"message": "SSH key deleted."}, status=200)


class SSHKeyInfoView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        logger.debug("SSHKeyInfoView GET called by user: %s", request.user)  # ✅ 로그 추가
        user = request.user
        return Response({
            "has_key": bool(user.encrypted_private_key),
            "encrypted_private_key_name": user.encrypted_private_key_name,
            "last_used_at": user.last_used_at
        })


class SSHKeyRetrieveView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        logger.info("SSHKeyRetrieveView GET called by user: %s", request.user)  # ✅ 로그 추가
        user = request.user
        if not user.encrypted_private_key:
            logger.warning("User tried to retrieve non-existent SSH key: %s", user)  # ✅ 로그 추가
            return Response({"error": "SSH 키가 등록되어 있지 않습니다."}, status=404)

        try:
            decrypted_key = user.decrypt_private_key()
            logger.info("SSH key successfully decrypted for user: %s", user)  # ✅ 로그 추가
            return Response({"ssh_key": decrypted_key})
        except Exception as e:
            logger.exception("SSH key 복호화 실패")  # ✅ 로그 추가
            return Response({"error": f"복호화 실패: {str(e)}"}, status=500)
